from __future__ import annotations

from collections.abc import AsyncIterator
from functools import lru_cache
from io import StringIO
import os
import textwrap
import typing
from typing import Iterable
from typing import Literal

from google.genai.client import AsyncClient
from google.genai.client import Client
from google.genai.types import FunctionDeclarationDict
from google.genai.types import SchemaDict
from google.genai.types import ToolDict
from google.genai.types import Type
from pydantic import Field
from pydantic import SecretStr

from tiny_gemini.utils import gemini_chunk_to_tiny_chunks
from tiny_gemini.utils import gemini_response_to_tiny_result
from tiny_gemini.utils import tiny_attributes_to_gemini_config
from tiny_gemini.utils import tiny_prompt_to_gemini_params
from tinygent.core.datamodels.llm import AbstractLLM
from tinygent.core.datamodels.llm import AbstractLLMConfig
from tinygent.core.datamodels.messages import AllTinyMessages
from tinygent.core.telemetry.decorators import tiny_trace
from tinygent.core.telemetry.otel import set_tiny_attribute
from tinygent.core.telemetry.utils import set_llm_telemetry_attributes
from tinygent.core.types.io.llm_io_chunks import TinyLLMResultChunk
from tinygent.llms.utils import accumulate_llm_chunks
from tinygent.llms.utils import group_chunks_for_telemetry

if typing.TYPE_CHECKING:
    from tinygent.core.datamodels.llm import LLMStructuredT
    from tinygent.core.datamodels.tool import AbstractTool
    from tinygent.core.types.io.llm_io_input import TinyLLMInput
    from tinygent.core.types.io.llm_io_result import TinyLLMResult


class GeminiLLMConfig(AbstractLLMConfig['GeminiLLM']):
    type: Literal['gemini'] = Field(default='gemini', frozen=True)

    model: str = Field(default='gemini-2.5-flash')

    temperature: float = Field(default=0.6)

    api_key: SecretStr | None = Field(
        default_factory=lambda: (
            SecretStr(os.environ['GEMINI_API_KEY'])
            if 'GEMINI_API_KEY' in os.environ
            else None
        ),
    )

    def build(self) -> GeminiLLM:
        return GeminiLLM(
            model=self.model,
            temperature=self.temperature,
            api_key=self.api_key.get_secret_value() if self.api_key else None,
        )


class GeminiLLM(AbstractLLM[GeminiLLMConfig]):
    def __init__(
        self,
        model: str = 'gemini-2.5-flash',
        temperature: float = 0.6,
        api_key: str | None = None,
    ) -> None:
        if not api_key and not (api_key := os.getenv('GEMINI_API_KEY')):
            raise ValueError(
                'Gemini API key must be provided either via config',
                " or 'GEMINI_API_KEY' env variable.",
            )

        self._sync_client: Client | None = None

        self.model = model
        self.temperature = temperature
        self.api_key = api_key

    @property
    def config(self) -> GeminiLLMConfig:
        return GeminiLLMConfig(
            model=self.model,
            temperature=self.temperature,
            api_key=SecretStr(self.api_key),
        )

    @property
    def supports_tool_calls(self) -> bool:
        return True

    def __get_sync_client(self) -> Client:
        if self._sync_client:
            return self._sync_client

        self._sync_client = Client(api_key=self.api_key)
        return self._sync_client

    def __get_async_client(self) -> AsyncClient:
        cli = self.__get_sync_client()
        return cli.aio

    def _tool_convertor(self, tool: AbstractTool) -> ToolDict:
        info = tool.info
        schema = info.input_schema

        def map_type(py_type: type) -> Type:
            mapping = {
                str: Type.STRING,
                int: Type.INTEGER,
                float: Type.NUMBER,
                bool: Type.BOOLEAN,
                list: Type.ARRAY,
                dict: Type.OBJECT,
            }
            return mapping.get(py_type, Type.STRING)  # default fallback

        properties = {}

        if schema:
            for name, field in schema.model_fields.items():
                field_type = (
                    field.annotation
                    if isinstance(field.annotation, type)
                    else type(field.annotation)
                )
                properties[name] = SchemaDict(
                    type=map_type(field_type),
                    description=field.description,
                )

        func_declaration = FunctionDeclarationDict(
            name=info.name,
            description=info.description,
            parameters=SchemaDict(
                type=Type.OBJECT,
                properties=properties,
                required=info.required_fields,
            ),
        )

        return ToolDict(function_declarations=[func_declaration])

    @lru_cache(maxsize=100_000)
    def _count_tokens(self, model: str, text: str) -> int:
        return (
            self.__get_sync_client()
            .models.count_tokens(model=model, contents=text)
            .total_tokens
            or 0
        )

    @tiny_trace()
    def generate_text(self, llm_input: TinyLLMInput) -> TinyLLMResult:
        params = tiny_prompt_to_gemini_params(llm_input)
        config = tiny_attributes_to_gemini_config(llm_input, self.temperature)

        chat = self.__get_sync_client().chats.create(
            model=self.model,
            config=config,
            history=params['history'],
        )
        res = chat.send_message(params['message'])  # type: ignore

        tiny_res = gemini_response_to_tiny_result(res)
        set_llm_telemetry_attributes(
            self.config, llm_input.messages, result=tiny_res.to_string()
        )
        return tiny_res

    @tiny_trace()
    async def agenerate_text(
        self,
        llm_input: TinyLLMInput,
    ) -> TinyLLMResult:
        params = tiny_prompt_to_gemini_params(llm_input)
        config = tiny_attributes_to_gemini_config(llm_input, self.temperature)

        chat = self.__get_async_client().chats.create(
            model=self.model,
            config=config,
            history=params['history'],
        )
        res = await chat.send_message(params['message'])  # type: ignore

        tiny_res = gemini_response_to_tiny_result(res)
        set_llm_telemetry_attributes(
            self.config, llm_input.messages, result=tiny_res.to_string()
        )
        return tiny_res

    @tiny_trace()
    async def stream_text(
        self, llm_input: TinyLLMInput
    ) -> AsyncIterator[TinyLLMResultChunk]:
        params = tiny_prompt_to_gemini_params(llm_input)
        config = tiny_attributes_to_gemini_config(llm_input, self.temperature)
        set_llm_telemetry_attributes(self.config, llm_input.messages)

        chat = self.__get_async_client().chats.create(
            model=self.model,
            config=config,
            history=params['history'],
        )
        res = await chat.send_message_stream(params['message'])  # type: ignore

        async def raw_chunks() -> AsyncIterator[TinyLLMResultChunk]:
            async for chunk in res:
                for tiny_chunk in gemini_chunk_to_tiny_chunks(chunk):
                    yield tiny_chunk

        accumulated_chunks: list[TinyLLMResultChunk] = []
        try:
            async for acc_chunk in accumulate_llm_chunks(raw_chunks()):
                accumulated_chunks.append(acc_chunk)
                yield acc_chunk
        finally:
            set_tiny_attribute(
                'result',
                group_chunks_for_telemetry(accumulated_chunks),
            )

    @tiny_trace()
    def generate_structured(
        self, llm_input: TinyLLMInput, output_schema: type[LLMStructuredT]
    ) -> LLMStructuredT:
        params = tiny_prompt_to_gemini_params(llm_input)
        config = tiny_attributes_to_gemini_config(
            llm_input,
            self.temperature,
            structured_output=output_schema,
        )

        chat = self.__get_sync_client().chats.create(
            model=self.model,
            config=config,
            history=params['history'],
        )
        res = chat.send_message(params['message'])  # type: ignore
        tiny_result = gemini_response_to_tiny_result(res)
        for message in tiny_result.tiny_iter():
            if content := getattr(message, 'content', None):
                try:
                    parsed = output_schema.model_validate_json(content)
                    set_llm_telemetry_attributes(
                        self.config,
                        llm_input.messages,
                        result=str(parsed),
                        output_schema=output_schema,
                    )
                    return parsed
                except Exception:
                    pass

        raise ValueError('No valid structured output found in Gemini response.')

    @tiny_trace()
    async def agenerate_structured(
        self, llm_input: TinyLLMInput, output_schema: type[LLMStructuredT]
    ) -> LLMStructuredT:
        params = tiny_prompt_to_gemini_params(llm_input)
        config = tiny_attributes_to_gemini_config(
            llm_input,
            self.temperature,
            structured_output=output_schema,
        )

        chat = self.__get_async_client().chats.create(
            model=self.model,
            config=config,
            history=params['history'],
        )
        res = await chat.send_message(params['message'])  # type: ignore
        tiny_result = gemini_response_to_tiny_result(res)
        for message in tiny_result.tiny_iter():
            if content := getattr(message, 'content', None):
                try:
                    parsed = output_schema.model_validate_json(content)
                    set_llm_telemetry_attributes(
                        self.config,
                        llm_input.messages,
                        result=str(parsed),
                        output_schema=output_schema,
                    )
                    return parsed
                except Exception:
                    pass

        raise ValueError('No valid structured output found in Gemini response.')

    @tiny_trace()
    def generate_with_tools(
        self, llm_input: TinyLLMInput, tools: list[AbstractTool]
    ) -> TinyLLMResult:
        gemini_tools = [self._tool_convertor(tool) for tool in tools]

        params = tiny_prompt_to_gemini_params(llm_input)
        config = tiny_attributes_to_gemini_config(
            llm_input,
            self.temperature,
            tools=gemini_tools,  # type: ignore
        )

        chat = self.__get_sync_client().chats.create(
            model=self.model,
            config=config,
            history=params['history'],
        )
        res = chat.send_message(params['message'])  # type: ignore

        tiny_res = gemini_response_to_tiny_result(res)
        set_llm_telemetry_attributes(
            self.config, llm_input.messages, result=tiny_res.to_string(), tools=tools
        )
        return tiny_res

    @tiny_trace()
    async def agenerate_with_tools(
        self, llm_input: TinyLLMInput, tools: list[AbstractTool]
    ) -> TinyLLMResult:
        gemini_tools = [self._tool_convertor(tool) for tool in tools]

        params = tiny_prompt_to_gemini_params(llm_input)
        config = tiny_attributes_to_gemini_config(
            llm_input,
            self.temperature,
            tools=gemini_tools,  # type: ignore
        )

        chat = self.__get_async_client().chats.create(
            model=self.model,
            config=config,
            history=params['history'],
        )
        res = await chat.send_message(params['message'])  # type: ignore

        tiny_res = gemini_response_to_tiny_result(res)
        set_llm_telemetry_attributes(
            self.config, llm_input.messages, result=tiny_res.to_string(), tools=tools
        )
        return tiny_res

    @tiny_trace()
    async def stream_with_tools(
        self, llm_input: TinyLLMInput, tools: list[AbstractTool]
    ) -> AsyncIterator[TinyLLMResultChunk]:
        gemini_tools = [self._tool_convertor(tool) for tool in tools]

        params = tiny_prompt_to_gemini_params(llm_input)
        config = tiny_attributes_to_gemini_config(
            llm_input,
            self.temperature,
            tools=gemini_tools,  # type: ignore
        )
        set_llm_telemetry_attributes(self.config, llm_input.messages, tools=tools)

        chat = self.__get_async_client().chats.create(
            model=self.model,
            config=config,
            history=params['history'],
        )
        res = await chat.send_message_stream(params['message'])  # type: ignore

        async def raw_chunks() -> AsyncIterator[TinyLLMResultChunk]:
            async for chunk in res:
                for tiny_chunk in gemini_chunk_to_tiny_chunks(chunk):
                    yield tiny_chunk

        accumulated_chunks: list[TinyLLMResultChunk] = []
        try:
            async for acc_chunk in accumulate_llm_chunks(raw_chunks()):
                accumulated_chunks.append(acc_chunk)
                yield acc_chunk
        finally:
            set_tiny_attribute(
                'result',
                group_chunks_for_telemetry(accumulated_chunks),
            )

    @tiny_trace()
    def count_tokens_in_messages(self, messages: Iterable[AllTinyMessages]) -> int:
        set_llm_telemetry_attributes(self.config, messages)

        number_of_tokens = sum(
            [self._count_tokens(self.model, m.tiny_str) for m in messages]
        )

        set_tiny_attribute('number_of_tokens', number_of_tokens)
        return number_of_tokens

    def __str__(self) -> str:
        buf = StringIO()

        buf.write('Gemini LLM Summary:\n')
        buf.write(textwrap.indent(f'Model: {self.model}\n', '\t'))
        buf.write(textwrap.indent(f'Temperature: {self.temperature}\n', '\t'))

        return buf.getvalue()



from .embedder import GeminiEmbedder
from .embedder import GeminiEmbedderConfig
from .llm import GeminiLLM
from .llm import GeminiLLMConfig

__all__ = [
    'GeminiEmbedder',
    'GeminiEmbedderConfig',
    'GeminiLLMConfig',
    'GeminiLLM',
]



from __future__ import annotations

import os
from typing import Literal

from google.genai.client import AsyncClient
from google.genai.client import Client
from pydantic import Field
from pydantic import SecretStr

from tinygent.core.datamodels.embedder import AbstractEmbedder
from tinygent.core.datamodels.embedder import AbstractEmbedderConfig
from tinygent.core.telemetry.decorators import tiny_trace
from tinygent.core.telemetry.utils import set_embedder_telemetry_attributes

_SUPPORTED_MODELS: dict[str, int] = {
    'gemini-embedding-001': 3072,
}


class GeminiEmbedderConfig(AbstractEmbedderConfig['GeminiEmbedder']):
    type: Literal['gemini'] = Field(default='gemini', frozen=True)

    model: str = Field(default='gemini-embedding-001')

    api_key: SecretStr | None = Field(
        default_factory=lambda: (
            SecretStr(os.environ['GEMINI_API_KEY'])
            if 'GEMINI_API_KEY' in os.environ
            else None
        ),
    )

    def build(self) -> GeminiEmbedder:
        return GeminiEmbedder(
            model=self.model,
            api_key=self.api_key.get_secret_value() if self.api_key else None,
        )


class GeminiEmbedder(AbstractEmbedder):
    def __init__(
        self,
        model: str = 'gemini-2.5-flash',
        api_key: str | None = None,
    ) -> None:
        if not api_key and not (api_key := os.getenv('GEMINI_API_KEY')):
            raise ValueError(
                'Gemini API key must be provided either via config',
                " or 'GEMINI_API_KEY' env variable.",
            )

        if model not in _SUPPORTED_MODELS:
            raise ValueError(
                f'Provided model name: {model} not in supported model names: {", ".join(_SUPPORTED_MODELS.keys())}'
            )

        self._sync_client: Client | None = Client(api_key=api_key)
        self._model = model

        self.api_key = api_key

    @property
    def model(self) -> str:
        return self._model

    @property
    def config(self) -> GeminiEmbedderConfig:
        return GeminiEmbedderConfig(
            model=self.model,
            api_key=SecretStr(self.api_key),
        )

    @property
    def embedding_dim(self) -> int:
        v = _SUPPORTED_MODELS.get(self.model)
        if not v:
            raise ValueError(
                f'Provided model name: {self.model} not in supported model names: {", ".join(_SUPPORTED_MODELS.keys())}'
            )
        return v

    def __get_sync_client(self) -> Client:
        if self._sync_client:
            return self._sync_client

        self._sync_client = Client(api_key=self.api_key)
        return self._sync_client

    def __get_async_client(self) -> AsyncClient:
        cli = self.__get_sync_client()
        return cli.aio

    @tiny_trace()
    def embed(self, query: str) -> list[float]:
        res = self.__get_sync_client().models.embed_content(
            model=self.model, contents=query
        )
        embedding = res.embeddings[0].values if res.embeddings else None
        if not embedding:
            raise ValueError(f'Error while creating embeddings for query {query}')

        set_embedder_telemetry_attributes(
            self.config,
            query,
            embedding_dim=self.embedding_dim,
            result_len=len(embedding),
        )
        return embedding

    @tiny_trace()
    def embed_batch(self, queries: list[str]) -> list[list[float]]:
        res = self.__get_sync_client().models.embed_content(
            model=self.model,
            contents=queries,  # type: ignore[arg-type]
        )

        embeddings = []
        for q, r in zip(queries, res.embeddings or [], strict=True):
            if (e := r.values) is None:
                raise ValueError(f'Error while creating embeddings for query {q}')
            embeddings.append(e)

        set_embedder_telemetry_attributes(
            self.config,
            queries,
            embedding_dim=self.embedding_dim,
            result_len=len(embeddings),
        )
        return embeddings

    @tiny_trace()
    async def aembed(self, query: str) -> list[float]:
        res = await self.__get_async_client().models.embed_content(
            model=self.model, contents=query
        )
        embedding = res.embeddings[0].values if res.embeddings else None
        if not embedding:
            raise ValueError(f'Error while creating embeddings for query {query}')

        set_embedder_telemetry_attributes(
            self.config,
            query,
            embedding_dim=self.embedding_dim,
            result_len=len(embedding),
        )
        return embedding

    @tiny_trace()
    async def aembed_batch(self, queries: list[str]) -> list[list[float]]:
        res = await self.__get_async_client().models.embed_content(
            model=self.model,
            contents=queries,  # type: ignore[arg-type]
        )

        embeddings = []
        for q, r in zip(queries, res.embeddings or [], strict=True):
            if (e := r.values) is None:
                raise ValueError(f'Error while creating embeddings for query {q}')
            embeddings.append(e)

        set_embedder_telemetry_attributes(
            self.config,
            queries,
            embedding_dim=self.embedding_dim,
            result_len=len(embeddings),
        )
        return embeddings



import typing
from typing import Any
from typing import cast

from google.genai.chats import GenerateContentResponse
from google.genai.types import AutomaticFunctionCallingConfigDict
from google.genai.types import Content
from google.genai.types import FunctionCall
from google.genai.types import FunctionCallingConfigDict
from google.genai.types import FunctionCallingConfigMode
from google.genai.types import FunctionResponse
from google.genai.types import GenerateContentConfigDict
from google.genai.types import ModelContent
from google.genai.types import Part
from google.genai.types import ToolConfigDict
from google.genai.types import ToolListUnionDict
from google.genai.types import UserContent
from langchain_core.messages import AIMessage
from langchain_core.outputs import ChatGeneration
from langchain_core.outputs import Generation

from tiny_gemini.types import GeminiParams
from tinygent.core.datamodels.llm import LLMStructuredT
from tinygent.core.datamodels.messages import TinyChatMessage
from tinygent.core.datamodels.messages import TinyChatMessageChunk
from tinygent.core.datamodels.messages import TinyHumanMessage
from tinygent.core.datamodels.messages import TinyPlanMessage
from tinygent.core.datamodels.messages import TinyReasoningMessage
from tinygent.core.datamodels.messages import TinySystemMessage
from tinygent.core.datamodels.messages import TinyToolCall
from tinygent.core.datamodels.messages import TinyToolResult
from tinygent.core.types.io.llm_io_chunks import TinyLLMResultChunk
from tinygent.core.types.io.llm_io_result import TinyLLMResult

if typing.TYPE_CHECKING:
    from tinygent.core.types.io.llm_io_input import TinyLLMInput


def _gemini_parts_to_text(parts: list[Part] | None) -> str:
    """Convert Gemini Parts to a single text string."""
    texts: list[str] = []
    for part in parts or []:
        if part.text:
            texts.append(part.text)
    return ''.join(texts)


def _gemini_extract_tool_calls(parts: list[Part] | None) -> list[dict[str, Any]]:
    """Extract tool calls from Gemini Parts."""
    tool_calls: list[dict[str, Any]] = []

    for part in parts or []:
        if fc := part.function_call:
            tool_calls.append(
                {'id': fc.id, 'name': fc.name, 'args': fc.args, 'type': 'tool_call'}
            )
    return tool_calls


def tiny_attributes_to_gemini_config(
    prompt: 'TinyLLMInput',
    temperature: float,
    tools: ToolListUnionDict | None = None,
    structured_output: type[LLMStructuredT] | None = None,
) -> GenerateContentConfigDict:
    conf_dict: GenerateContentConfigDict = {}
    conf_dict['temperature'] = temperature

    if tools:
        conf_dict['tools'] = tools
        conf_dict['tool_config'] = ToolConfigDict(
            function_calling_config=FunctionCallingConfigDict(
                mode=FunctionCallingConfigMode.AUTO
            )
        )

    if structured_output:
        conf_dict['response_mime_type'] = 'application/json'
        conf_dict['response_json_schema'] = structured_output.model_json_schema()
        conf_dict['automatic_function_calling'] = AutomaticFunctionCallingConfigDict(
            disable=True
        )
        conf_dict['tool_config'] = ToolConfigDict(
            function_calling_config=FunctionCallingConfigDict(
                mode=FunctionCallingConfigMode.NONE
            )
        )
        conf_dict.pop('tools', None)

    for msg in prompt.messages:
        if isinstance(msg, TinySystemMessage):
            conf_dict['system_instruction'] = msg.content

    return conf_dict


def tiny_prompt_to_gemini_params(
    prompt: 'TinyLLMInput',
) -> GeminiParams:
    """Convert TinyLLMInput to Gemini GenerateContent parameters."""
    params: list[Content] = []

    for msg in prompt.messages:
        if isinstance(msg, TinyHumanMessage):
            params.append(UserContent(str(msg.content)))

        elif isinstance(msg, TinySystemMessage):
            pass  # INFO: Handled in config_dict

        elif isinstance(msg, TinyChatMessage):
            params.append(ModelContent(str(msg.content)))

        elif isinstance(msg, TinyPlanMessage):
            params.append(ModelContent(f'<PLAN>\n{msg.content}\n</PLAN>'))

        elif isinstance(msg, TinyReasoningMessage):
            params.append(ModelContent(f'<REASONING>\n{msg.content}\n</REASONING>'))

        elif isinstance(msg, TinyToolCall):
            params.append(
                ModelContent(
                    parts=[
                        Part(
                            function_call=FunctionCall(
                                id=msg.call_id or 'tool_call_1',
                                name=msg.tool_name,
                                args=msg.arguments,
                            )
                        )
                    ]
                )
            )

        elif isinstance(msg, TinyToolResult):
            params.append(
                ModelContent(
                    parts=[
                        Part(
                            function_response=FunctionResponse(
                                id=msg.call_id or 'tool_call_1',
                                name=msg.raw.info.name,
                                response={'tool_result': msg.content},
                            )
                        )
                    ]
                )
            )

        else:
            raise TypeError(f'Unsupported TinyMessage type: {type(msg)}')

    message = params[-1].parts
    history = params[:-1]

    message = cast(list[Part], message)
    history = cast(list[Content], history)

    return GeminiParams(history=history, message=message)  # type: ignore


def gemini_response_to_tiny_result(resp: GenerateContentResponse) -> TinyLLMResult:
    """Convert Gemini GenerateContentResponse to TinyLLMResult."""
    generations: list[list[Generation]] = []

    for candidate in resp.candidates or []:
        if not (content := candidate.content):
            continue

        additional_kwargs = {}

        text = _gemini_parts_to_text(content.parts)
        tool_calls = _gemini_extract_tool_calls(content.parts)

        ai_msg = AIMessage(content=text)

        if tool_calls:
            additional_kwargs['tool_calls'] = tool_calls

            ai_msg.additional_kwargs = additional_kwargs
            ai_msg.tool_calls = cast(list[Any], tool_calls)

        generations.append([ChatGeneration(message=ai_msg, text=text)])

    llm_output = {
        'id': resp.response_id or 'response_id_missing',
        'model': resp.model_version or 'model_missing',
        'created': resp.create_time,
        'usage': resp.usage_metadata.model_dump() if resp.usage_metadata else None,
    }

    return TinyLLMResult(
        generations=generations,
        llm_output=llm_output,
    )


def gemini_chunk_to_tiny_chunks(
    chunk: GenerateContentResponse,
) -> list[TinyLLMResultChunk]:
    """Convert Gemini GenerateContentResponse chunk to TinyLLMResultChunk."""
    chunks: list[TinyLLMResultChunk] = []

    tiny_result = gemini_response_to_tiny_result(chunk)
    for msg in tiny_result.tiny_iter():
        if hasattr(msg, 'content') and isinstance(msg, TinyChatMessage):
            chunks.append(
                TinyLLMResultChunk(
                    type='message',
                    message=TinyChatMessageChunk(
                        content=msg.content,
                        metadata=msg.metadata,
                    ),
                )
            )
            continue

        if isinstance(msg, TinyToolCall):
            chunks.append(
                TinyLLMResultChunk(
                    type='tool_call',
                    full_tool_call=msg,
                    metadata=msg.metadata,
                )
            )
            continue

        if isinstance(msg, TinyToolResult):
            chunks.append(
                TinyLLMResultChunk(
                    type='message',
                    message=TinyChatMessageChunk(
                        content=msg.content,
                        metadata=msg.metadata,
                    ),
                )
            )
            continue

        if isinstance(msg, (TinyPlanMessage, TinyReasoningMessage)):
            chunks.append(
                TinyLLMResultChunk(
                    type='message',
                    message=TinyChatMessageChunk(
                        content=msg.content,
                        metadata=msg.metadata,
                    ),
                )
            )
            continue

    return chunks



from typing import TypedDict

from google.genai.types import ContentOrDict
from google.genai.types import Part


class GeminiParams(TypedDict):
    message: list[Part]
    history: list[ContentOrDict]



def _register_gemini() -> None:
    from tinygent.core.runtime.global_registry import GlobalRegistry

    from .llm import GeminiLLM
    from .llm import GeminiLLMConfig

    registry = GlobalRegistry().get_registry()

    registry.register_llm('gemini', GeminiLLMConfig, GeminiLLM)


_register_gemini()
